@@ -100,7 +100,8 @@ cat("\n")
     else
       mess="Dimension out of range"
 
-    retour <- list(neval=ret$neval, nregions=ret$nregions,
+    retour <- list(method="divonne",
+                   neval=ret$neval, nregions=ret$nregions,
                    ifail=ret$fail, value=ret$integral,
                 abs.error=ret$error,
                    neval=ret$neval,
